# -*- coding: utf-8 -*-

import logging


class BasicJob(object):
    def __init__(self, name=None, schedule=None, is_enabled=False):
        self._logger = logging.getLogger()
        self._name = name
        self._schedule = schedule
        self._is_enabled = is_enabled

    def __call__(self):
        pass

    @property
    def name(self):
        return self._name

    @property
    def schedule(self):
        return self._schedule

    @property
    def is_enabled(self):
        return self._is_enabled



# -*- coding: utf-8 -*-

from datetime import datetime
from datetime import time


class MarketUtils(object):
    OPEN_TIME = time(9, 25)
    CLOSE_TIME = time(15)

    @classmethod
    def is_opening(cls, datetime_=None):
        if datetime_ is None:
            datetime_ = datetime.now()

        if datetime_.isoweekday() not in range(1, 6):
            return False
        if datetime_.time() <= cls.OPEN_TIME or datetime_.time() >= cls.CLOSE_TIME:
            return False
        return True

    @classmethod
    def is_closed(cls, datetime_=None):
        return not cls.is_opening(datetime_)



# -*- coding: utf-8 -*-

import unittest

from shipane_sdk.stock import StockUtils


class StockUtilsTest(unittest.TestCase):
    def test_new_stocks(self):
        df = StockUtils.new_stocks()
        self.assertTrue((df.columns == ['code', 'xcode', 'name', 'ipo_date', 'price']).all())



# -*- coding: utf-8 -*-

import collections
import inspect
import logging
import os
import unittest

import six
from six.moves import configparser

if six.PY2:
    ConfigParser = configparser.RawConfigParser
else:
    ConfigParser = configparser.ConfigParser

from shipane_sdk import JoinQuantExecutor


class JoinQuantExecutorTest(unittest.TestCase):
    def setUp(self):
        logging.basicConfig(level=logging.INFO)

        config = ConfigParser()
        dir_path = os.path.dirname(os.path.realpath(__file__))
        config.read('{}/../../config/config.ini'.format(dir_path))
        self.Order = collections.namedtuple('Order', ['is_buy', 'security', 'price', 'amount'])
        self.executor = JoinQuantExecutor(host=config.get('ShiPanE', 'host'))

    def test_buy_stock(self):
        mock_order = self.Order
        mock_order.is_buy = True
        mock_order.order_id = 1
        mock_order.security = '000001.XSHE'
        mock_order.price = 11.11
        mock_order.amount = 100
        response = self.executor.execute(mock_order)
        print(inspect.stack()[0][3] + ' - ' + response.text)
        json = response.json();
        if response.status_code == 200:
            self.assertTrue(json['id'])
        elif response.status_code == 400:
            self.assertTrue(json['message'])
        else:
            self.fail()

    def test_sell_stock(self):
        mock_order = self.Order
        mock_order.is_buy = False
        mock_order.order_id = 2
        mock_order.security = '000001.XSHE'
        mock_order.price = 11.11
        mock_order.amount = 100
        response = self.executor.execute(mock_order)
        print(inspect.stack()[0][3] + ' - ' + response.text)
        json = response.json();
        if response.status_code == 200:
            self.assertTrue(json['id'])
        elif response.status_code == 400:
            self.assertTrue(json['message'])
        else:
            self.fail()

    def test_cancel(self):
        mock_order = self.Order
        mock_order.is_buy = True
        mock_order.order_id = 3
        mock_order.security = '000001.XSHE'
        mock_order.price = 9.01
        mock_order.amount = 100
        self.executor.execute(mock_order)

        response = self.executor.cancel(mock_order)
        print(inspect.stack()[0][3] + ' - ' + response.text)
        if response.status_code != 200:
            self.fail()



# 可以自己import我们平台支持的第三方python模块，比如pandas、numpy等。
import shipane_sdk


# 在这个方法中编写任何的初始化逻辑。context对象将会在你的算法策略的任何方法之间做传递。
def init(context):
    context.s1 = "000001.XSHE"
    # 实时打印日志
    logger.info("Interested at stock: " + str(context.s1))

def before_trading(context):
    # 创建 RiceQuantExecutor 对象
    # 可选参数包括：host, port, key, client, timeout 等
    # 请将下面的 IP 替换为实际 IP
    context.__executor = shipane_sdk.RiceQuantExecutor(
        host='xxx.xxx.xxx.xxx',
        port=8888,
        key='',
        client=''
    )

# 你选择的证券的数据更新将会触发此段逻辑，例如日或分钟历史数据切片或者是实时数据切片更新
def handle_bar(context, bar_dict):
    # 开始编写你的主要的算法逻辑

    # bar_dict[order_book_id] 可以拿到某个证券的bar信息
    # context.portfolio 可以拿到现在的投资组合状态信息

    # 使用order_shares(id_or_ins, amount)方法进行落单

    # TODO: 开始编写你的算法吧！
    # 保存 order_id
    order_id = order_shares(context.s1, 100)
    # 实盘易依据 order_id 下单
    context.__executor.execute(order_id)

    order_id = order_shares(context.s1, -100)
    context.__executor.execute(order_id)

    cancel_order(order_id)
    context.__executor.cancel(order_id)



# -*- coding: utf-8 -*-
import codecs
import collections
import distutils.util
import logging
import os
import os.path
import time

import six
from apscheduler.schedulers.background import BackgroundScheduler
from six.moves import configparser

from shipane_sdk import Client
from shipane_sdk.ap import APCronParser
from shipane_sdk.jobs.new_stock_purchase import NewStockPurchaseJob
from shipane_sdk.jobs.online_quant_following import OnlineQuantFollowingJob
from shipane_sdk.joinquant.client import JoinQuantClient
from shipane_sdk.ricequant.client import RiceQuantClient

if six.PY2:
    ConfigParser = configparser.RawConfigParser
else:
    ConfigParser = configparser.ConfigParser


class Scheduler(object):
    def __init__(self):
        logging.basicConfig(level=logging.INFO, format='%(asctime)-15s %(levelname)-6s %(message)s')
        self._logger = logging.getLogger()

        config_path = os.path.join(os.path.expanduser('~'), '.shipane_sdk', 'config', 'scheduler.ini')
        self._logger.info('Config path: %s', config_path)
        self._config = ConfigParser()
        self._config.readfp(codecs.open(config_path, "r", "utf_8_sig"))

        self._scheduler = BackgroundScheduler()
        self._client = Client(self._logger, **dict(self._config.items('ShiPanE')))

    def start(self):
        self.__add_job(self.__create_new_stock_purchase_job())
        self.__add_job(self.__create_join_quant_following_job())
        self.__add_job(self.__create_rice_quant_following_job())

        self._scheduler.start()
        print('Press Ctrl+{0} to exit'.format('Break' if os.name == 'nt' else 'C'))

        try:
            while True:
                time.sleep(1)
        except (KeyboardInterrupt, SystemExit):
            self._scheduler.shutdown()

    def __add_job(self, job):
        if job.is_enabled:
            self._scheduler.add_job(job, APCronParser.parse(job.schedule), name=job.name, misfire_grace_time=None)
        else:
            self._logger.warning('{} is not enabled'.format(job.name))

    def __create_new_stock_purchase_job(self):
        section = 'NewStocks'
        options = self.__build_options(section)
        client_aliases = self.__filter_client_aliases(section)
        return NewStockPurchaseJob(self._client, client_aliases, '{}FollowingJob'.format(section), **options)

    def __create_join_quant_following_job(self):
        section = 'JoinQuant'
        options = self.__build_options(section)
        client_aliases = self.__filter_client_aliases(section)
        quant_client = JoinQuantClient(**options)
        return OnlineQuantFollowingJob(self._client, quant_client, client_aliases, '{}FollowingJob'.format(section),
                                       **options)

    def __create_rice_quant_following_job(self):
        section = 'RiceQuant'
        options = self.__build_options(section)
        client_aliases = self.__filter_client_aliases(section)
        quant_client = RiceQuantClient(**options)
        return OnlineQuantFollowingJob(self._client, quant_client, client_aliases, '{}FollowingJob'.format(section),
                                       **options)

    def __build_options(self, section):
        if not self._config.has_section(section):
            return dict()

        options = dict(self._config.items(section))
        options['enabled'] = bool(distutils.util.strtobool(options['enabled']))
        return options

    def __filter_client_aliases(self, section):
        all_client_aliases = dict(self._config.items('ClientAliases'))
        client_aliases = map(str.strip, filter(None, self._config.get(section, 'clients', fallback='').split(',')))
        return collections.OrderedDict(
            (client_alias, all_client_aliases[client_alias]) for client_alias in client_aliases)



# -*- coding: utf-8 -*-

import os
import unittest

import six
from six.moves import configparser

if six.PY2:
    ConfigParser = configparser.RawConfigParser
else:
    ConfigParser = configparser.ConfigParser

from shipane_sdk.joinquant.client import JoinQuantClient


class JoinQuantClientTest(unittest.TestCase):
    def setUp(self):
        config = ConfigParser()
        dir_path = os.path.dirname(os.path.realpath(__file__))
        config.read('{}/../../config/config.ini'.format(dir_path))
        self._jq_client = JoinQuantClient(username=config.get('JoinQuant', 'username'),
                                          password=config.get('JoinQuant', 'password'),
                                          backtest_id=config.get('JoinQuant', 'backtestId'))

    def test_query(self):
        self._jq_client.login()
        transactions = self._jq_client.query()
        self.assertTrue(isinstance(transactions, list))



# -*- coding: utf-8 -*-

import datetime

try:
    from shipane_sdk.client import Client
except:
    pass

try:
    from kuanke.user_space_api import *
except:
    pass


def warning(self, msg, *args, **kwargs):
    self.warn(msg, *args, **kwargs)


class JoinQuantExecutor(object):
    def __init__(self, **kwargs):
        if 'log' in globals():
            import types
            if not callable(getattr(log, 'warning', None)):
                log.warning = types.MethodType(warning, log)
            self._logger = log
        else:
            import logging
            self._logger = logging.getLogger()
        self._client = Client(self._logger, **kwargs)
        self._client_param = kwargs.get('client')
        self._order_id_map = dict()
        self._expire_before = datetime.datetime.combine(datetime.date.today(), datetime.time.min)

    @property
    def client(self):
        return self._client

    def execute(self, order):
        self._logger.info("[实盘易] 跟单：" + str(order))

        if order is None:
            self._logger.info('[实盘易] 委托为空，忽略下单请求')
            return
        if self.__is_expired(order):
            self._logger.info('[实盘易] 委托已过期，忽略下单请求')
            return

        try:
            action = 'BUY' if order.is_buy else 'SELL'
            order_type = 'LIMIT' if order.limit > 0 else 'MARKET'
            price_type = 0 if order_type == 'LIMIT' else 4
            actual_order = self._client.execute(self._client_param,
                                                action=action,
                                                symbol=order.security,
                                                type=order_type,
                                                priceType=price_type,
                                                price=order.limit,
                                                amount=order.amount)
            self._order_id_map[order.order_id] = actual_order['id']
            return actual_order
        except Exception as e:
            self._logger.error("[实盘易] 下单异常：" + str(e))

    def cancel(self, order):
        if order is None:
            self._logger.info('[实盘易] 委托为空，忽略撤单请求')
            return

        try:
            order_id = order if isinstance(order, int) else order.order_id
            if order_id in self._order_id_map:
                self._client.cancel(self._client_param, self._order_id_map[order_id])
            else:
                self._logger.warning('[实盘易] 未找到对应的委托编号')
        except Exception as e:
            self._logger.error("[实盘易] 撤单异常：" + str(e))

    def __is_expired(self, order):
        return order.add_time < self._expire_before



# -*- coding: utf-8 -*-
import json
from datetime import datetime

from shipane_sdk.jobs.basic_job import BasicJob
from shipane_sdk.stock import *


class NewStockPurchaseJob(BasicJob):
    def __init__(self, client, client_aliases=None, name=None, **kwargs):
        super(NewStockPurchaseJob, self).__init__(name, kwargs.get('schedule', None), kwargs.get('enabled', False))

        self._client = client
        self._client_aliases = client_aliases

    def __call__(self):
        today = datetime.strftime(datetime.today(), '%Y-%m-%d')
        df = StockUtils.new_stocks()
        df = df[(df.ipo_date == today)]
        for client_alias in self._client_aliases:
            client = self._client_aliases[client_alias]
            self._logger.info(u'客户端[%s(%s)]开始新股申购', client_alias, client)
            for index, row in df.iterrows():
                try:
                    order = {
                        'symbol': row['xcode'], 'type': 'LIMIT', 'price': row['price'], 'amountProportion': 'ALL'
                    }
                    self._logger.info(u'下单：%s', json.dumps(order))
                    self._client.buy(client, **order)
                except Exception as e:
                    self._logger.exception('账户[%s(%s)]申购新股[%s（%s）]失败', client_alias, client, row['name'], row['code'])
            self._logger.info(u'客户端[%s(%s)]结束新股申购', client_alias, client)



# -*- coding: utf-8 -*-

import copy
import re

import pandas as pd
import requests
from requests import Request
from six.moves.urllib.parse import urlencode


class Client(object):
    KEY_REGEX = r'key=([^&]*)'

    def __init__(self, logger=None, **kwargs):
        self._logger = logger
        self._host = kwargs.pop('host', 'localhost')
        self._port = kwargs.pop('port', 8888)
        self._key = kwargs.pop('key', '')
        self._timeout = kwargs.pop('timeout', (5.0, 10.0))

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, value):
        self._host = value

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, value):
        self._port = value

    @property
    def key(self):
        return self._key

    @key.setter
    def key(self, value):
        self._key = value

    @property
    def timeout(self):
        return self._timeout

    @timeout.setter
    def timeout(self, value):
        self._timeout = value

    def get_account(self, client=None, timeout=None):
        request = Request('GET', self.__create_url(client, 'accounts'))
        response = self.__send_request(request, timeout)
        return response.json()

    def get_positions(self, client=None, timeout=None):
        request = Request('GET', self.__create_url(client, 'positions'))
        response = self.__send_request(request, timeout)
        json = response.json()
        sub_accounts = pd.DataFrame(json['subAccounts']).T
        positions = pd.DataFrame(json['dataTable']['rows'], columns=json['dataTable']['columns'])
        return {'sub_accounts': sub_accounts, 'positions': positions}

    def buy(self, client=None, timeout=None, **kwargs):
        kwargs['action'] = 'BUY'
        return self.__execute(client, timeout, **kwargs)

    def sell(self, client=None, timeout=None, **kwargs):
        kwargs['action'] = 'SELL'
        return self.__execute(client, timeout, **kwargs)

    def execute(self, client=None, timeout=None, **kwargs):
        return self.__execute(client, timeout, **kwargs)

    def cancel(self, client=None, order_id=None, timeout=None):
        request = Request('DELETE', self.__create_order_url(client, order_id))
        self.__send_request(request, timeout)

    def cancel_all(self, client=None, timeout=None):
        request = Request('DELETE', self.__create_order_url(client))
        self.__send_request(request, timeout)

    def query(self, client=None, navigation=None, timeout=None):
        request = Request('GET', self.__create_url(client, '', navigation=navigation))
        response = self.__send_request(request, timeout)
        json = response.json()
        df = pd.DataFrame(json['dataTable']['rows'], columns=json['dataTable']['columns'])
        return df

    def start_clients(self, timeout=None):
        request = Request('PUT', self.__create_url(None, 'clients'))
        self.__send_request(request, timeout)

    def shutdown_clients(self, timeout=None):
        request = Request('DELETE', self.__create_url(None, 'clients'))
        self.__send_request(request, timeout)

    def __execute(self, client=None, timeout=None, **kwargs):
        if not kwargs.get('type'):
            kwargs['type'] = 'LIMIT'
        request = Request('POST', self.__create_order_url(client), json=kwargs)
        response = self.__send_request(request)
        return response.json()

    def __create_order_url(self, client=None, order_id=None, **params):
        return self.__create_url(client, 'orders', order_id, **params)

    def __create_url(self, client, resource, resource_id=None, **params):
        all_params = copy.deepcopy(params)
        if client is not None:
            all_params.update(client=client)
        all_params.update(key=self._key)
        if resource_id is None:
            path = '/{}'.format(resource)
        else:
            path = '/{}/{}'.format(resource, resource_id)

        return '{}{}?{}'.format(self.__create_base_url(), path, urlencode(all_params))

    def __create_base_url(self):
        return 'http://' + self._host + ':' + str(self._port)

    def __send_request(self, request, timeout=None):
        prepared_request = request.prepare()
        timeout = timeout if timeout is not None else self._timeout
        self.__log_request(prepared_request)
        with requests.sessions.Session() as session:
            response = session.send(prepared_request, timeout=timeout)
        self.__log_response(response)
        response.raise_for_status()
        return response

    def __log_request(self, prepared_request):
        if self._logger is None:
            return

        url = self.__eliminate_privacy(prepared_request.path_url)
        if prepared_request.body is None:
            self._logger.info('Request:\n{} {}'.format(prepared_request.method, url))
        else:
            self._logger.info('Request:\n{} {}\n{}'.format(prepared_request.method, url, prepared_request.body))

    def __log_response(self, response):
        if self._logger is None:
            return

        message = 'Response:\n{} {}\n{}'.format(response.status_code, response.reason, response.text)
        if response.status_code == 200:
            self._logger.info(message)
        else:
            self._logger.error(message)

    @classmethod
    def __eliminate_privacy(cls, url):
        match = re.search(cls.KEY_REGEX, url)
        key = match.group(1)
        masked_key = '*' * len(key)
        url = re.sub(cls.KEY_REGEX, "key={}".format(masked_key), url)
        return url



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from codecs import open
from os import path
import os.path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='shipane_sdk',

    version='1.0.0.b2',

    description=u'实盘易（ShiPanE）Python SDK，通达信自动化交易 API。',
    long_description=long_description,

    url='https://github.com/sinall/ShiPanE-Python-SDK',

    author='sinall',
    author_email='gaoruinan@163.com',

    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Intended Audience :: Financial and Insurance Industry',
        'Topic :: Office/Business :: Financial :: Investment',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='ShiPanE SDK 通达信 TDX Automation',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=['requests', 'six', 'apscheduler', 'lxml', 'cssselect', 'bs4', 'html5lib', 'pandas', 'rqopen-client'],

    extras_require={
        'dev': [],
        'test': [],
    },

    package_data={
    },

    data_files=[(os.path.join(os.path.expanduser('~'), '.shipane_sdk', 'config'), ['config/scheduler-example.ini'])],

    scripts=['scripts/shipane-scheduler.py'],

    entry_points={
        'console_scripts': [
        ],
    },
)



# -*- coding: utf-8 -*-

import logging
import os
import unittest

import six
from requests import HTTPError
from six.moves import configparser

from shipane_sdk import Client

if six.PY2:
    ConfigParser = configparser.RawConfigParser
else:
    ConfigParser = configparser.ConfigParser


class ClientTest(unittest.TestCase):
    def setUp(self):
        logging.basicConfig(level=logging.DEBUG)
        config = ConfigParser()
        dir_path = os.path.dirname(os.path.realpath(__file__))
        config.read('{}/../config/config.ini'.format(dir_path))
        self.client = Client(logging.getLogger(), host=config.get('ShiPanE', 'host'))
        self.client.start_clients()

    def test_get_account(self):
        try:
            self.client.get_account()
        except HTTPError as e:
            self.fail()

    def test_get_positions(self):
        try:
            data = self.client.get_positions()
            sub_accounts = data['sub_accounts']
            self.assertGreater(sub_accounts['总资产']['人民币'], 0)
            positions = data['positions']
            self.assertIsNotNone(positions['证券代码'][0])
        except HTTPError as e:
            self.fail()

    def test_buy_stock(self):
        try:
            order = self.client.buy(symbol='000001', price=8.11, amount=100)
            self.assertIsNotNone(order['id'])
        except HTTPError as e:
            result = e.response.json()
            self.assertIsNotNone(result['message'])

    def test_sell_stock(self):
        try:
            order = self.client.sell(symbol='000001', price=9.51, amount=100)
            self.assertIsNotNone(order['id'])
        except HTTPError as e:
            result = e.response.json()
            self.assertIsNotNone(result['message'])

    def test_cancel_all(self):
        try:
            self.client.cancel_all()
        except HTTPError as e:
            self.fail()

    def test_query(self):
        try:
            df = self.client.query(None, '查询>资金股份')
            self.assertIsNotNone(df['证券代码'][0])
        except HTTPError as e:
            self.fail()
